// (C) 2019-2025 GoodData Corporation
import { AbstractExecutionFactory } from "../toolkit/execution.js";
import { defFingerprint, defWithDimensions, defWithSorting, defWithDateFormat, defWithExecConfig, defWithBuckets, } from "@gooddata/sdk-model";
import { NotSupported, NotImplemented, } from "@gooddata/sdk-backend-spi";
import isEqual from "lodash/isEqual.js";
/**
 * @internal
 */
export class CustomExecutionFactory extends AbstractExecutionFactory {
    config;
    state;
    constructor(workspace, config, state) {
        super(workspace);
        this.config = config;
        this.state = state;
    }
    forDefinition = (def) => {
        return new CustomPreparedExecution(def, this, this.config, this.state);
    };
}
/**
 * @internal
 */
class CustomPreparedExecution {
    definition;
    executionFactory;
    config;
    state;
    _fingerprint;
    constructor(definition, executionFactory, config, state) {
        this.definition = definition;
        this.executionFactory = executionFactory;
        this.config = config;
        this.state = state;
    }
    execute = () => {
        const { authApiCall } = this.state;
        return authApiCall((client) => {
            const context = {
                config: this.config,
                execution: this,
                resultFactory: this.resultFactory,
                state: this.state,
                client,
            };
            return this.config.resultProvider(context);
        });
    };
    explain = () => {
        console.warn("Backend does not support explain mode");
        return {
            data: () => Promise.reject(new Error(`Backend does not support explain mode data call.`)),
            download: () => Promise.resolve(),
        };
    };
    withDimensions = (...dimsOrGen) => {
        return this.executionFactory.forDefinition(defWithDimensions(this.definition, ...dimsOrGen));
    };
    withSorting = (...items) => {
        return this.executionFactory.forDefinition(defWithSorting(this.definition, items));
    };
    withBuckets = (...buckets) => {
        return this.executionFactory.forDefinition(defWithBuckets(this.definition, ...buckets));
    };
    withDateFormat = (dateFormat) => {
        return this.executionFactory.forDefinition(defWithDateFormat(this.definition, dateFormat));
    };
    withExecConfig = (config) => {
        return this.executionFactory.forDefinition(defWithExecConfig(this.definition, config));
    };
    withSignal = (_signal) => {
        throw new NotSupported("Cancelling is not supported by the custom backend.");
    };
    equals = (other) => {
        return isEqual(this.definition, other.definition);
    };
    fingerprint = () => {
        if (!this._fingerprint) {
            this._fingerprint = defFingerprint(this.definition);
        }
        return this._fingerprint;
    };
    resultFactory = (dimensions, fingerprint) => {
        return new CustomExecutionResult(dimensions, fingerprint, this, this.config, this.state);
    };
}
/**
 * @internal
 */
class CustomExecutionResult {
    dimensions;
    _fingerprint;
    execution;
    config;
    state;
    definition;
    constructor(dimensions, _fingerprint, execution, config, state) {
        this.dimensions = dimensions;
        this._fingerprint = _fingerprint;
        this.execution = execution;
        this.config = config;
        this.state = state;
        this.definition = execution.definition;
    }
    readAll = () => {
        return this.state.authApiCall((client) => {
            if (!this.config.dataProvider) {
                throw new NotImplemented("custom backend does not specify dataProvider");
            }
            const context = {
                config: this.config,
                state: this.state,
                result: this,
                client,
            };
            return this.config.dataProvider(context);
        });
    };
    readForecastAll() {
        throw new NotSupported("Forecasting is not supported by the custom backend.");
    }
    readAnomalyDetectionAll() {
        throw new NotSupported("Anomaly detection is not supported by the custom backend.");
    }
    readClusteringAll() {
        throw new NotSupported("Clustering is not supported by the custom backend.");
    }
    readWindow = (offset, size) => {
        return this.state.authApiCall((client) => {
            if (!this.config.dataProvider) {
                throw new NotImplemented("custom backend does not specify dataProvider");
            }
            const context = {
                config: this.config,
                state: this.state,
                result: this,
                window: {
                    offset,
                    size,
                },
                client,
            };
            return this.config.dataProvider(context);
        });
    };
    transform = () => {
        return this.execution;
    };
    equals = (other) => {
        return this._fingerprint === other.fingerprint();
    };
    fingerprint = () => {
        return this._fingerprint;
    };
    export = (_options) => {
        throw new NotSupported("exports from custom backend are not supported");
    };
}
//# sourceMappingURL=execution.js.map